"""
This module contains the default values used by os-torado.
"""

LOG_ENABLED = True
LOG_LEVEL = 'DEBUG'
LOG_FORMAT = '[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s'
LOG_DATEFORMAT = '%Y-%m-%d %H:%M:%S'

HTTP_PORT = 8080
BIND_ADDRESS = ''
DEFAULT_HOST = ''
